#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_turbo_markdown
----------------------------------

Tests for `turbo_markdown` module.
"""


import sys
import unittest
from contextlib import contextmanager
from click.testing import CliRunner

from turbo_markdown import turbo_markdown
from turbo_markdown import cli



class TestTurbo_markdown(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_something(self):
        pass

    def test_command_line_interface(self):
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'turbo_markdown.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output


# -*- coding: utf-8 -*-

import socket
import platform
import os

import os.path
import click
from turbo_markdown import run_server


def port_is_used(port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    if sock.connect_ex(('localhost', port)) == 0:
        return True

    return False


@click.command()
def main(args=None):
    """Console script for turbo_markdown"""
    abspath = os.path.abspath('.')
    port = 8888
    while True:
        if not port_is_used(port):
            break
        port += 1
    url = 'http://localhost:%s' % port
    if platform.system() == 'Darwin':
        os.system('open %s' % url)
    else:
        click.echo("Open your brower visit %s" % url)
    run_server(abspath, port)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import os


def get_tree(docs_path):
    doc_dict = {}
    # current relative path start from root path
    scan_file(docs_path, doc_dict, parent_relative_path='/')
    return doc_dict


def generate_html_tree(docs_path, doc_dict):
    html_tree = generate_html(docs_path, doc_dict)
    return html_tree


def generate_html(parent_path, doc_dict, hide=False):
    html = '<ul class="nav-list" >'
    if hide:
        html = '<ul class="nav-list sub-nav" style="display:none;" >'

    parent = doc_dict[parent_path]
    parent_content = parent['content']
    parent_relative_path = parent['relative_path']

    for current in parent_content:
        current_path = os.path.join(parent_path, current)
        if current_path not in doc_dict:
            continue
        current_content = doc_dict[current_path]['content']
        if isinstance(current_content, list):
            sub_html = generate_html(current_path, doc_dict, True)
            if sub_html != '<ul class="nav-list" ></ul>':
                html += '<li class="has-sub" ><a class="js-has-sub close" href="javascript:void(0)" >%s</a>%s</li>' % (current, sub_html)   # noqa
        else:
            if current.endswith('.md') or current.endswith('.markdown'):
                real_path = os.path.realpath(
                    os.path.join(parent_relative_path, current))
                html += '''<li><a href="%s" >%s</a></li>''' % (
                    real_path, current)

    html += '</ul>'

    return html


def scan_file(parent_path, doc_dict, parent_relative_path):
    all_docs = os.listdir(parent_path)
    doc_dict[parent_path] = {
        'content': all_docs,
        'relative_path': parent_relative_path,
    }
    for current in all_docs:
        current_path = os.path.join(parent_path, current)
        if os.path.isdir(
            current_path) and not os.path.basename(
                current_path).startswith('.'):
            scan_file(
                current_path,
                doc_dict,
                os.path.join(parent_relative_path, current))
        else:
            if current.endswith('.md') or current.endswith('.markdown'):
                doc_dict[current_path] = {
                    'content': current,
                    'relative_path': parent_relative_path,
                }


def sort_path(c1, c2):
    """
    目录层级排序，
    如果目录层级一样，文件名或目录名排序
    """
    result = cmp(c1.count('/'), c2.count('/'))
    if result == 0:
        return cmp(c1, c2)
    else:
        return result


def print_fold_tree(doc_dict=None):
    if doc_dict:
        keys = doc_dict.keys()
        keys.sort(sort_path)
        for k in keys:
            if isinstance(doc_dict[k], list):
                print 'folder', '-->', k, '-->', doc_dict[k]
            else:
                print 'file', '-->', k, '-->', doc_dict[k]


if __name__ == '__main__':
    doc_dict = get_tree()
    print generate_html_tree(doc_dict)



# -*- coding:utf-8 -*-
from __future__ import absolute_import, division, print_function, with_statement

import os
import codecs

# from pygments.lexers import get_lexer_for_filename
# from pygments.formatters.html import HtmlFormatter
# from pygments import highlight
# from pygments.styles import get_style_by_name
import mistune
from turbo import app, register
from turbo.conf import app_config
import tornado.template

from turbo_markdown.util import get_tree, generate_html_tree
from turbo_markdown.template import template_html

app_config.app_name = 'turbo-markdown'
app_config.web_application_setting = {
    'xsrf_cookies': False,
    'cookie_secret': 'abcdefg',
    'static_path': os.path.join(os.path.dirname(__file__), "static"),
}

DOC_DICT = None
DOCS_PATH = ''
HTML_TREE = None


def parse_markdown(path):
    current_file = os.path.join(DOCS_PATH, path)
    if path.endswith('.md') or path.endswith('.markdown'):
        with codecs.open(current_file, mode='r', encoding='utf-8') as f:
            html = mistune.markdown(f.read())

        return html
    else:
        return ''
        # basename = os.path.basename(current_file)
        # with codecs.open(current_file, mode='r', encoding='utf-8') as f:
        #     return highlight(
        #         f.read(),
        #         get_lexer_for_filename(basename),
        #         HtmlFormatter(style=get_style_by_name('colorful')))


class HomeHandler(app.BaseHandler):

    def get(self, *path):
        self.template_path = ''
        global DOC_DICT
        global HTML_TREE
        html = ''
        if path and path[0]:
            html = parse_markdown(path[0])

        if 'X-Requested-With' in self.request.headers:
            return self.write(html)

        if not DOC_DICT:
            DOC_DICT = get_tree(DOCS_PATH)

        if not HTML_TREE:
            HTML_TREE = generate_html_tree(DOCS_PATH, DOC_DICT)
        t = tornado.template.Template(template_html)
        self.write(t.generate(
            html_tree=HTML_TREE, html=html, title=os.path.basename(DOCS_PATH)))


def run_server(docs_path='', port=None):
    if not docs_path:
        print('No Path Found')
        return
    global DOCS_PATH
    DOCS_PATH = docs_path
    register.register_url('/', HomeHandler)
    register.register_url('/(.*)', HomeHandler)
    app.start(port)


if __name__ == '__main__':
    run_server()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'turbo>=0.4.5',
    'mistune',
]

test_requirements = [
]

setup(
    name='turbo_markdown',
    version='0.5.0',
    description="A cli tool to view local markdown files",
    long_description=readme + '\n\n' + history,
    author="wecatch",
    author_email='zhyq0826@126.com',
    url='https://github.com/wecatch/turbo_markdown',
    packages=[
        'turbo_markdown',
    ],
    package_dir={'turbo_markdown':
                 'turbo_markdown'},
    entry_points={
        'console_scripts': [
            'turbo-markdown=turbo_markdown.cli:main'
        ]
    },
    include_package_data=True,
    package_data={'turbo_markdown': [
        'static/js/*', 'static/css/*', 'static/img/*']},
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='markdown,turbo',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



import hashlib
import os.path


TEMPLATE_HTML = """
<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>{{title}}</title>
  <link rel="stylesheet" href="/static/css/github.css?v=css_version">
  <script type="text/javascript" src="/static/js/jquery-3.2.1.min.js" ></script>
</head>
<body>
  <div class="doc-tree js-doc-tree" >
    {% raw html_tree %}
  </div>
  <div class="ui grid main-wrapper" >
    <div class="row" >
        <div class="doc-content js-doc-content codehilite" >
          {% raw html %}
        </div>
    </div>
  </div>

  <script type="text/javascript">
    var gDocCache = {};

    $("a.js-has-sub").on("click",function (e) {
      e.preventDefault();
      $(this).toggleClass("close")
      $(this).parent().find("ul:first").slideToggle();
    });

    $("div.js-doc-tree").on("click","a",function(e){
      e.preventDefault();
      var $this = $(this);
      if($this.hasClass("js-has-sub")) return;

      var path = $this.attr("href");
      var title = $this.html();
      document.title = title;
      pushHistoryState(path, title, path);
      fetchHtmlContent(path);
    });


    function pushHistoryState(state, title, url) {
      if (window.history.pushState) {
        history.pushState(state, title, url);
      } else {
        window.location.href = window.location.host + path;
      }
    }

    function fetchHtmlContent(path){
      if(!path)return;
      var key = path.replace(/\//gi,'_');
      var cacheData = gDocCache[key];
      var docContent = $("div.js-doc-content");
      if(cacheData){
        docContent.html(cacheData);
        return;
      }

      $.get(path).done(function(data){
        docContent.html(data);
        gDocCache[key] = data;
      }).fail(function(){
        docContent.html("something wrong, please try");
      });
    }

    onpopstate = function(e){
      fetchHtmlContent(e.state);
    }
  </script>
</body>
</html>
"""


css_v = hashlib.md5(
    open(os.path.join(
        os.path.dirname(__file__),
        'static/css/github.css'), 'r').read()).hexdigest()[0:6]

template_html = TEMPLATE_HTML.replace('css_version', css_v)
